(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/fire/database'), require('@angular/core'), require('akita-ng-fire'), require('rxjs/operators')) :
    typeof define === 'function' && define.amd ? define('akita-ng-fire/rtdb', ['exports', '@angular/fire/database', '@angular/core', 'akita-ng-fire', 'rxjs/operators'], factory) :
    (global = global || self, factory((global['akita-ng-fire'] = global['akita-ng-fire'] || {}, global['akita-ng-fire'].rtdb = {}), global.ng.fire.database, global.ng.core, global['akita-ng-fire'], global.rxjs.operators));
}(this, (function (exports, database, core, akitaNgFire, operators) { 'use strict';

    /** Set the configuration for the collection service */
    function RealTimeConfig(options) {
        if (options === void 0) { options = {}; }
        return function (constructor) {
            Object.keys(options).forEach(function (key) { return constructor[key] = options[key]; });
        };
    }

    var RealTimeService = /** @class */ (function () {
        function RealTimeService(store, path, rtdb) {
            this.store = store;
            try {
                this.rtdb = rtdb || core.inject(database.AngularFireDatabase);
            }
            catch (err) {
                throw new Error('RealTimeService requires AngularFireDatabase.');
            }
            this.nodePath = path;
            this.listRef = this.rtdb.list(this.path);
        }
        Object.defineProperty(RealTimeService.prototype, "path", {
            get: function () {
                return this.constructor['nodeName'] || this.nodePath;
            },
            enumerable: false,
            configurable: true
        });
        Object.defineProperty(RealTimeService.prototype, "idKey", {
            get: function () {
                return this.store ? this.store.idKey : 'id';
            },
            enumerable: false,
            configurable: true
        });
        /**
         * Function triggered when adding/updating data to the database
         * @note should be overridden
         */
        RealTimeService.prototype.formatToDatabase = function (entity) {
            return entity;
        };
        /**
         * Function triggered when getting data from database
         * @note should be overridden
         */
        RealTimeService.prototype.formatFromDatabase = function (entity) {
            return entity;
        };
        /**
         * @description just sync with node you specified when initialized the service class without updating the store
         */
        RealTimeService.prototype.syncNode = function () {
            var _this = this;
            return this.listRef.valueChanges().pipe(operators.map(function (value) { return _this.formatFromDatabase(value); }));
        };
        /**
         * @description sync the node with the store. `formatFromDatabase` will be called every time there is data incoming.
         */
        RealTimeService.prototype.syncNodeWithStore = function () {
            var _this = this;
            return this.listRef.stateChanges().pipe(operators.tap(function (data) {
                switch (data.type) {
                    case 'child_added': {
                        akitaNgFire.upsertStoreEntity(_this.store.storeName, _this.formatFromDatabase(data.payload.toJSON()), data.key);
                        break;
                    }
                    case 'child_removed': {
                        akitaNgFire.removeStoreEntity(_this.store.storeName, data.key);
                        break;
                    }
                    case 'child_changed': {
                        akitaNgFire.upsertStoreEntity(_this.store.storeName, _this.formatFromDatabase(data.payload.toJSON()), data.key);
                    }
                }
            }));
        };
        /**
         * @description adds a store entity to your database.
         * @param entity to add.
         */
        RealTimeService.prototype.add = function (entity) {
            var _a;
            var _this = this;
            if (entity[this.idKey]) {
                return this.rtdb.database.ref(this.nodePath + '/' + entity[this.idKey])
                    .set(this.formatToDatabase(entity), function (error) {
                    if (error) {
                        throw error;
                    }
                });
            }
            if (Array.isArray(entity)) {
                var ids_1 = [];
                var promises = entity.map(function (e) {
                    var _a;
                    var id = _this.rtdb.createPushId();
                    ids_1.push(id);
                    return _this.listRef.set(id, Object.assign(Object.assign({}, e), (_a = {}, _a[_this.idKey] = id, _a)));
                });
                return Promise.all(promises).then(function () { return ids_1; });
            }
            else {
                var id_1 = this.rtdb.createPushId();
                return this.listRef.set(id_1, this.formatToDatabase(Object.assign(Object.assign({}, entity), (_a = {}, _a[this.idKey] = id_1, _a)))).then(function () { return id_1; });
            }
        };
        RealTimeService.prototype.update = function (idOrEntity, entity) {
            var _this = this;
            if (Array.isArray(idOrEntity)) {
                return Promise.all(idOrEntity.map(function (e) { return _this.listRef.update(e[_this.idKey], _this.formatToDatabase(e)); }));
            }
            else {
                if (typeof idOrEntity === 'string') {
                    return this.listRef.update(idOrEntity, this.formatToDatabase(entity));
                }
                else if (typeof idOrEntity === 'object') {
                    var id = idOrEntity[this.idKey];
                    return this.listRef.update(id, this.formatToDatabase(idOrEntity));
                }
                else {
                    return new Error("Couldn't find corresponding entity/ies: " + idOrEntity + ", " + entity);
                }
            }
        };
        /**
         *
         * @param id of the entity to remove
         */
        RealTimeService.prototype.remove = function (id) {
            try {
                return this.listRef.remove(id);
            }
            catch (error) {
                return new Error("Error while removing entity with this id: " + id + ". " + error);
            }
        };
        /**
         * @warn clears the node and every child of the node.
         */
        RealTimeService.prototype.clearNode = function () {
            return this.rtdb.database.ref(this.nodePath).remove();
        };
        return RealTimeService;
    }());

    /**
     * Generated bundle index. Do not edit.
     */

    exports.RealTimeConfig = RealTimeConfig;
    exports.RealTimeService = RealTimeService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=akita-ng-fire-rtdb.umd.js.map
